package jp.sourceforge.jirc;

import java.awt.*;
import java.awt.event.*;
//import java.awt.font.*;

import java.util.*;

import jp.sourceforge.jirc.net.*;
import jp.sourceforge.jirc.text.*;

class AWTFrame extends Frame implements ClientComponent {
    private TextField tf = new TextField();
    private TextArea listener;
    private Choice choice = new Choice();
    private Hashtable channelTable = new Hashtable();

    private CardLayout cardLayout = new CardLayout();
    private Panel cardPanel = new Panel(cardLayout);

    private Window window;
    private AWTMenuBar menuBar;

    private final String applName = "jIRC";
    private final String CRLF = "\r\n";

    AWTFrame(Window window) {
    		super();
    		this.window = window;
    		
//    		listener.setFont(new Font("monaco", Font.PLAIN, 13));
    		listener = getChannelTextArea("", 5, 10, TextArea.SCROLLBARS_VERTICAL_ONLY);

    		menuBar = new AWTMenuBar(window);
    		setMenuBar(menuBar.getMenuBar());
	
    		choice.addItemListener(new ItemListenerImpl());

    		super.addWindowListener(new WindowClosingListener());
    }

    public void init(Container container) {
		Panel panel = new Panel(new BorderLayout());
    		panel.add(tf, BorderLayout.CENTER);
    		panel.add(choice, BorderLayout.WEST);
    		panel.add(listener, BorderLayout.SOUTH);

    		container.add(cardPanel, BorderLayout.CENTER);
    		container.add(panel, BorderLayout.SOUTH);
    		container.setVisible(true);
    		
    		tf.requestFocus();
    		tf.addActionListener(new TextFieldInputListener());
    }
    
    TextArea getChannelTextArea(String string, int i, int j, int k) {
		TextArea ta = new TextArea(string, i, j, k);
		ta.setEditable(false);
		ta.addKeyListener(getKeyAdapter(ta));
		ta.addInputMethodListener(new InputMethodCancelListener(ta));
		
		return ta;
    }
    
    private class InputMethodCancelListener implements InputMethodListener {
    		TextComponent textComponent;
    		InputMethodCancelListener(TextComponent tc) {
    			this.textComponent = tc;    			
    		}
    		
    		public void inputMethodTextChanged(InputMethodEvent e) {
    			//		    System.out.println(e);
    			if(textComponent.getSelectedText() == null) {
    				tf.requestFocus();
    			}
    		}

    		public void caretPositionChanged(InputMethodEvent e) {
    			//		    System.out.println(e);
    		}
    }

    protected void append(String string) {
    		listener.append(string);
    }
    
    public void append(Channel channel, String string) {
		if (channel == null
				|| !channel.equals(window.getNowChannel())
				|| channel.toString().equals(channel.getIRC().getServer())) {
			append(string + CRLF);
		}
    }

    public void exit() {
    		window.exit();
    }

    KeyAdapter getKeyAdapter(TextArea ta) {
    		final TextArea jta = ta;
    		KeyAdapter adapter = new KeyAdapter() {
    			public void keyPressed(KeyEvent e) {
    				if(jta.getSelectedText() == null) {
    					tf.requestFocus();
    				}
    			}
		
    			public void keyTyped(KeyEvent e) {
    				tf.requestFocus();
    			}
	    };

	    return adapter;
    }

    TextField getTextField() {
    		return tf;
    }
    
    public void setTopic() {
    		ChildWindow cw = (ChildWindow )window.getNowChannel();
		cw.setTopic();
    }

    public void setTopic(String string) {
    		setTitle(string + applName);
    }

    Container getContentPane() {
    		return this;
    }

    public void setDefaultCloseOperation(int i) {
	//	setDefaultCloseOperation(i);
    }


    public Channel makeChannel(IRC irc, String name) {
    		AWTChannel cw = new AWTChannel(name, irc, this);
    		addChannel(cw);

    		return cw;
    }

    public Channel makeServerChannel(IRC irc, String name) {
    		return makeChannel(irc, name);
    }

    private void addChannel(AWTChannel cw) {
    		cardPanel.add(cw.toString() +cw.getIRC().getDate(), cw.getComponent());
    		channelTable.put(cw.toString(), cw);
    		choice.add(cw.toString());
    		choice.select(choice.getItemCount() - 1);
    		setNowChannel((Channel )cw);
    		tf.requestFocus();
    }

    private void setNowChannel(String name) {
    		setNowChannel((Channel )channelTable.get(name));
    }

    public void setNowChannel(Channel channel) {
    		/*
    		 System.out.println(channel.toString());
    		 */
    		cardLayout.show(cardPanel,
    						channel.toString() + channel.getIRC().getDate());
    		window.setNowChannel(channel);
    		setTopic();
    }

    public void writeMessage(String channel, String message) {
    		window.writeMessage(channel, message);
    }

    public void setConnection(IRC irc, boolean flag) {
    		menuBar.setConnected(flag);
    }
    
    public void addDefaultWindowListener() {
    }

    public void changeUser(IRC irc, String oldNick, String newNick) {
		String[] args = { oldNick, newNick};
		String message = IRCMessageFormat.getNickMessage(args);
    	
		Enumeration channels = irc.getChannels();

		while(channels.hasMoreElements()) {
			AWTChannel channel = (AWTChannel )channels.nextElement();

			if(channel.toString().equals(oldNick)) {
				irc.removeChannel(channel);
				channel.setChannelName(newNick);
				irc.addChannel(channel);
				addChannel(channel);
				channel.appendCommandMessage(message);
			} else {
				channel.renameNick(oldNick, newNick, message);
			}
		}
    }

    boolean removeUser(Channel channel, String nick) {
    		AWTChannel awtChannel = (AWTChannel )channel;
    		return awtChannel.isRemoveUser(nick);
    }
    
    public void removeUser(IRC irc, String nick, String message) {
		Enumeration channels = irc.getChannels();

		while(channels.hasMoreElements()) {
			Channel channel = (Channel )channels.nextElement();

			if(channel.equals(nick) || removeUser(channel, nick)) {
				String[] args = {nick, message};
				channel.appendCommandMessage(IRCMessageFormat.getQuitMessage(args));
			}
		}
    }

    public void showNextChannel() {
		int i = choice.getSelectedIndex();

		if(i != -1 && !cardPanel.hasFocus()) {
			int c = choice.getItemCount();
			if(i < c - 1) {
				choice.select(i + 1);
			} else {
				choice.select(0);
			}

			setNowChannel(choice.getSelectedItem());
			tf.requestFocus();
		}
	}
    
    public void removeChannel(Channel channel) {
    		AWTChannel cw = (AWTChannel )channelTable.get(channel.toString());
    		cardLayout.removeLayoutComponent(cw.getComponent());
    		channelTable.remove(cw.toString());

    		choice.remove(channel.toString());
    		choice.select(choice.getItemCount() - 1);
    		Channel ch = (Channel )channelTable.get(choice.getSelectedItem());
    		setNowChannel(ch.toString());
    }

    private class ItemListenerImpl implements ItemListener {
    		public void itemStateChanged(ItemEvent e) {
    			setNowChannel(e.getItem().toString());
    			tf.requestFocus();
    		}
    }

    private class WindowClosingListener extends WindowAdapter {
    		public void windowClosing(WindowEvent e) {
    			exit();
    		}
    }
    
    private class TextFieldInputListener implements ActionListener {
        private final char LF = '\n';

        TextFieldInputListener() {
        }

        public void actionPerformed(ActionEvent ae) {
        		String string = tf.getText();

        		if(!string.equals("")) {
        			if(string.indexOf(LF) > 0) {
        				StringTokenizer st = new StringTokenizer(string, "" + LF);
        				while(st.hasMoreTokens()) {
        					window.getNowChannel().writeMessage(st.nextToken());
        				}
        			} else {
        				window.getNowChannel().writeMessage(string);
        			}
        		} else {
        			showNextChannel();
        		}
        		
        		tf.setText("");
        }
    }
}
