package jp.sourceforge.jirc;

import java.awt.*;
import java.awt.event.InputMethodEvent;
import java.awt.event.InputMethodListener;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.tree.*;

import jp.sourceforge.jirc.net.IRC;

class SwingChannel extends ChildWindow {
    private boolean barLock = false;

    private JPanel panel;
    private JScrollBar scrollBar;
    private JTextArea ta;
    
    private DefaultMutableTreeNode treeNode;

    private SwingFrame window;

    SwingChannel(String name, IRC irc, SwingFrame win) {
		super(name, irc, win);
		panel = new JPanel(new BorderLayout());

    		this.window = win;

    		ta = new JTextArea(10, 30);
    		ta.setLineWrap(true);
    		//	ta.setWrapStyleWord(true);
    		ta.setEditable(false);

    		JScrollPane scrollPane = new JScrollPane(ta,
    				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
					JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

    		scrollBar = scrollPane.getVerticalScrollBar();

    		ChangeListener l = new ChangeListener() {
    			public void stateChanged(ChangeEvent e) {
    				if(barLock) {
    					scrollBar.setValue(scrollBar.getMaximum());
    				}
    				barLock = false;
    			}
	    };

	    scrollBar.getModel().addChangeListener(l);
	    panel.add(scrollPane, BorderLayout.CENTER);
	    ta.addKeyListener(window.getKeyAdapter(ta));

	    ta.addInputMethodListener(new InputMethodListener() {
	    		public void inputMethodTextChanged(InputMethodEvent e) {
	    			//		    System.out.println(e);
	    			if(ta.getSelectedText() == null) {
	    				window.getJTextField().requestFocus();
	    			}
	    		}
		
	    		public void caretPositionChanged(InputMethodEvent e) {
	    			//		    System.out.println(e);
	    		}
	    });
	    
	    treeNode = new DefaultMutableTreeNode(this);
    }
    
    DefaultMutableTreeNode getTreeNode() {
    		return treeNode;
    }

    Component getComponent() {
		return (Component )panel;
    }

    public void addUser(String nick) {
		window.addUser(this, nick);
    }

    public void removeUser(String nick) {
		window.removeUser(this, nick);
    }

    public void setChanop(String nick, boolean flag) {
		window.setChanop(this, nick, flag);
    }

    public void setVoice(String nick, boolean flag) {
		window.setVoice(this, nick, flag);
    }

    public void append(String message) {
    		if((scrollBar.getValue() + scrollBar.getVisibleAmount()) 
    				== scrollBar.getMaximum()) {
    			barLock = true;
    		} else {
    			barLock = false;
    		}
    		ta.setForeground(Color.BLACK);
    		ta.append(message + CRLF);
    }
    
    public void appendCommandMessage(String message) {
//		ta.setForeground(Color.GREEN);
		ta.append(getTime() + " " + message + CRLF);
    }
    
    public void appendErrorMessage(String message) {
//		ta.setForeground(Color.RED);
		ta.append(getTime() + " " + message + CRLF);
    }
}
