package jp.sourceforge.jirc;

import java.util.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import jp.sourceforge.jirc.net.*;
import jp.sourceforge.jirc.text.*;

class SwingFrame extends JFrame implements ClientComponent {
    private SwingChannel listenTA;
    private JTextField tf = new JTextField();
    private JSplitPane splitPane;
    private JScrollBar scrollBar;
    private JTree tree;

    private CardLayout cardLayout = new CardLayout();
    private JPanel cardPanel = new JPanel(cardLayout);

    private WindowTreeModel windowTreeModel = new WindowTreeModel(this);
    private Window window;
    private SwingMenuBar menuBar;

    private Vector channelList = new Vector();
    private final String applName = "jIRC";
    private final String CRLF = "\r\n";

    SwingFrame(Window window) {
    		this.window = window;
    		menuBar = new SwingMenuBar(window);
    		listenTA = new SwingChannel(null, null, this);
    }

    public void init(Container container) {
    		setJMenuBar(menuBar.getJMenuBar());
	    
    		this.tree = new JTree(windowTreeModel);
    		tree.addTreeSelectionListener(new TreeSelectionListenerImpl());
    		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
    		tree.setRootVisible(false);
    		JScrollPane treePane = new JScrollPane(tree,
    				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

    		JPanel panel = new JPanel(new BorderLayout());
    		panel.add(tf, BorderLayout.NORTH);
    		panel.add(listenTA.getComponent(), BorderLayout.CENTER);

    		splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
    				cardPanel, panel);

    		container.add(splitPane, BorderLayout.CENTER);
    		container.add(treePane, BorderLayout.WEST);
    		container.setVisible(true);

    		tf.requestFocus();
    		tf.addActionListener(new InputListener(window, tf));
    }

    protected void append(String string) {
    		listenTA.append(string);
    }
    
    public void append(Channel channel, String string) {
		if (channel == null || !channel.equals(window.getNowChannel())) {
			append(string + CRLF);
		}
    }

    public void exit() {
    		window.exit();
    }

    JTextField getJTextField() {
    		return tf;
    }

    KeyAdapter getKeyAdapter(JTextArea ta) {
    		final JTextArea jta = ta;
    		KeyAdapter adapter = new KeyAdapter() {
    			public void keyPressed(KeyEvent e) {
    				if(jta.getSelectedText() == null) {
    					tf.requestFocus();
    				}
    			}
		
    			public void keyTyped(KeyEvent e) {
    				tf.requestFocus();
    			}
	    };

	    	return adapter;
    }

    private class WindowListenerImpl extends WindowAdapter {
    		public void windowClosed(WindowEvent we) {
    			exit();
    		}
    }
    
    private class TreeSelectionListenerImpl implements TreeSelectionListener {
    		public void valueChanged(TreeSelectionEvent e) {
    			TreePath path = e.getNewLeadSelectionPath();
    			if(path == null) {
    				path = e.getOldLeadSelectionPath();
    			}
	    
    			Object channel = null;
    			int count = path.getPathCount();
    			if(count < 4) { // getPathCount == 0 is null
    				channel = path.getPath()[path.getPathCount() - 1];
    			} else { // server
    				channel = path.getPath()[1];
    			}

	    // count == 0 unknown
	    //       == 1 
	    //       == 2 is server
	    //       == 3 is channel
	    //       == 4 is nick
    			if(count < 4) {
    				DefaultMutableTreeNode dmt = (DefaultMutableTreeNode )channel;
    				Channel nowChannel = (Channel )dmt.getUserObject();
    				setNowChannel(nowChannel);
    			}
    			tf.requestFocus();
    		}
    }
    
    public Channel makeServerChannel(IRC irc, String name) {
    		SwingChannel cw = new SwingChannel(name, irc, this);
    		windowTreeModel.removeParentWindow();
    		windowTreeModel.addParentWindow(cw);
    		addChannel(cw);

    		return cw;
    }

    public void changeUser(IRC irc, String oldNick, String newNick) {
    		Enumeration channels = irc.getChannels();

    		while(channels.hasMoreElements()) {
    			SwingChannel channel = (SwingChannel )channels.nextElement();
    			MutableTreeNode treeNode = channel.getTreeNode();

    			if(channel.equals(oldNick)) {
    				SwingChannel window = (SwingChannel )channel;

    				irc.removeChannel(channel);
    				window.setChannelName(newNick);
    				irc.addChannel(channel);
    				windowTreeModel.addChannel(window);
    				addChannel(window);

    				String[] args = { oldNick, newNick};
    				channel.appendCommandMessage(IRCMessageFormat.getNickMessage(args));
    			} else if(treeNode != null) {
    				MutableTreeNode childNode = windowTreeModel.getUserNode(treeNode, oldNick);
    				if(childNode != null) {
    					char head = childNode.toString().charAt(0);
    					String fullNewNick = newNick;
    					if(head == '@' || head == '+') {
    						fullNewNick = head + newNick;
    					}
    					windowTreeModel.changeUserNode(treeNode, childNode, fullNewNick);
    					String[] args = { oldNick, newNick};
    					channel.appendCommandMessage(IRCMessageFormat.getNickMessage(args));
    				}
    			}
    		}
    }

    public void removeUser(IRC irc, String nick, String message) {
    		Enumeration channels = irc.getChannels();

    		while(channels.hasMoreElements()) {
    			Channel channel = (Channel )channels.nextElement();

    			if(channel.equals(nick) || removeUser(channel, nick)) {
    				String[] args = {nick, message};
    				channel.appendCommandMessage(IRCMessageFormat.getQuitMessage(args));
    			}
    		}
    }
    
    public void addChannel(SwingChannel cw) {
    		channelList.add(cw);
    		cardPanel.add(cw.toString() +cw.getIRC().getDate(), cw.getComponent());
    		windowTreeModel.reload();
    		setNowChannel(cw);
    }

    public void removeChannel(Channel channel) {
    		windowTreeModel.removeChannel((SwingChannel )channel);
    }
    
    void removeWindow(SwingChannel cw) {
    		channelList.remove(cw);
    		cardLayout.removeLayoutComponent(cw.getComponent());
    		setNowChannel((ChildWindow )channelList.lastElement());
    }

    public void setNowChannel(Channel channel) {
    		cardLayout.show(cardPanel, channel.toString() + channel.getIRC().getDate());
    		window.setNowChannel(channel);
    		setTopic();
    		tree.expandRow(0);
    }

    private Channel getNowServer() {
    		return windowTreeModel.getNowServer();
    }

    Channel getNowChannel() {
    		return window.getNowChannel();
    }

    public void addUser(Channel channel, String nick) {
    		windowTreeModel.addUser((SwingChannel )channel, nick);
    }
    
    boolean removeUser(Channel channel, String nick) {
		return windowTreeModel.removeUser((SwingChannel )channel, nick);
    }
    
    public Channel makeChannel(IRC irc, String name) {
    		SwingChannel cw = new SwingChannel(name, irc, this);
    		windowTreeModel.addChannel(cw);
    		addChannel(cw);

    		return cw;
    }

    void setChanop(Channel channel, String nick, boolean flag) {
    		windowTreeModel.setUserMode((SwingChannel )channel, nick, "@", flag);
    }

    void setVoice(Channel channel, String nick, boolean flag) {
    		SwingChannel swingChannel = (SwingChannel )channel;
    		MutableTreeNode childNode = windowTreeModel.getUserNode(swingChannel.getTreeNode(), nick);
    		char head = childNode.toString().charAt(0);
    		if(head != '@') {
    			windowTreeModel.setUserMode((SwingChannel )channel, nick, "+", flag);
    		}
    }
    
    public void setTopic() {
    		ChildWindow cw = (ChildWindow )window.getNowChannel();
    		cw.setTopic();
    }

    	public void setTopic(String string) {
    		setTitle(string + applName);
    }

    public void addDefaultWindowListener() {
    		addWindowListener((WindowListener )new WindowListenerImpl());
    }

    public void writeMessage(String channel, String message) {
    		window.writeMessage(channel, message);
    }

    public void setConnection(IRC irc, boolean flag) {
    		if(window.getIRC().equals(irc)) {
    			menuBar.setConnected(flag);
    		}
    }
}
