package jp.sourceforge.jirc;

import java.io.UnsupportedEncodingException;
import java.util.StringTokenizer;

import javax.swing.*;

class SwingMenuBar extends AbstractMenuBar {
    private JMenuBar menuBar = new JMenuBar();
    private JMenuItem connectionItem = new JMenuItem(getMessage("connection"));
    private JMenuItem disconnectionItem = new JMenuItem(getMessage("disconnection"));
    
    private JMenu commandMenu = null;
    private JMenu channelMenu = null;
    //    private JMenu operMenu = null;
    
    private boolean hasMenu = false;
    
    SwingMenuBar(Window window) {
    		super(window);
	
    		if(!hasMenu) {
    			commandMenu = getCommandMenu();
    			channelMenu = getChannelMenu();
    			//	    operMenu = getOperMenu();
	    
    			menuBar.add(getFileMenu());

    			JMenu menu = getEncodingMenu();
    			//	    if(menu != null) {
    			//		add(menu);
    			//	    }
    			//	    add(commandMenu);

    			menuBar.add(channelMenu);
    			//	    add(operMenu);
    			hasMenu = true;
    		}
    }

    JMenuBar getJMenuBar() {
    		return menuBar;
    }
    
    private JMenu getFileMenu() {
    		JMenu menu = new JMenu(getMessage("file"));
    		JMenuItem quitItem = new JMenuItem(getMessage("quit"));

    		menu.add(connectionItem);
    		menu.add(disconnectionItem);
    		menu.add(new JSeparator());
    		setMenuItem(menu, getMessage("preferences")).addActionListener(new PreferencesActionImpl());
    		menu.add(new JSeparator()); 
    		setMenuItem(menu, getMessage("quit")).addActionListener(new QuitActionImpl());
	
    		connectionItem.addActionListener(new ConnectionActionImpl());
    		disconnectionItem.addActionListener(new DisconnectionActionImpl());
	
    		disconnectionItem.setEnabled(false);
	
    		return menu;
    }

    private JMenu getEncodingMenu() {
    		JMenu menu = null;
    		String encodingName  = getMessage("encodingName");
    		String encodingNames = getMessage("encodingNames");

    		if(!encodingName.equals(encodingNames)) {
    			ButtonGroup group = null;

    			try {
    				encodingName.getBytes(encodingName);

    				group = new ButtonGroup();
    				menu = new JMenu(getMessage("encoding"));
    				setEncGroupMenuItem(menu, group, encodingName, true);
    			} catch(UnsupportedEncodingException e) {
    				JOptionPane.showMessageDialog(null, e.getLocalizedMessage());
    				return null;
    			}

    			if(encodingNames.indexOf(",") > 0) {
    				StringTokenizer st = new StringTokenizer(encodingNames, ",");

    				String targetEncodingName;
    				while(st.hasMoreTokens()) {
    					targetEncodingName = st.nextToken();
    					if(!targetEncodingName.equals(encodingName)) {
    						try {
    							targetEncodingName.getBytes(targetEncodingName);
    							setEncGroupMenuItem(menu, group, targetEncodingName, false);
    						} catch(UnsupportedEncodingException e) {
    							JOptionPane.showMessageDialog(null, e.getLocalizedMessage());
    						}
    					}
    				}
    			} else {
    				try {
    					encodingNames.getBytes(encodingNames);
    					setEncGroupMenuItem(menu, group, encodingNames, false);
    				} catch(UnsupportedEncodingException e) {
    					JOptionPane.showMessageDialog(null, e.getLocalizedMessage());
    					menu = null;
    					return null;
    				}
    			}
    		}

    		return menu;
    }

    private JRadioButtonMenuItem setEncGroupMenuItem(JMenu menu,
						     ButtonGroup group, String text, boolean selected) {
    		JRadioButtonMenuItem item = new JRadioButtonMenuItem(text, selected);
    		menu.add(item);
    		group.add(item);
    		item.addActionListener(new SelectedActionImpl(text));

    		return item;
    }
    
    private JMenu getCommandMenu() {
    		JMenu menu = new JMenu(getMessage("command"));

    		//	setMenuItem(menu, "OPER");
    		//	setMenuItem(menu, "MODE");
    		setMenuItem(menu, "NICK...").addActionListener(new NickActionImpl());;
    		setMenuItem(menu, "JOIN...").addActionListener(new JoinActionImpl());;
    		//	setMenuItem(menu, "NAMES");
    		//	setMenuItem(menu, "LIST");
    		//	setMenuItem(menu, "PRIVMSG");
    		//	setMenuItem(menu, "NOTICE");
    		//	menu.add(new JSeparator());
    		//	setMenuItem(menu, "MOTD");
    		//	setMenuItem(menu, "LUSERS");
    		//	setMenuItem(menu, "VERSION");
    		//	setMenuItem(menu, "STATS");
    		//	setMenuItem(menu, "LINKS");
    		//	setMenuItem(menu, "TIME");
    		//	setMenuItem(menu, "CONNECT");
    		//	setMenuItem(menu, "TRACE");
    		//	setMenuItem(menu, "ADMIN");
    		//	setMenuItem(menu, "INFO");
    		//	menu.add(new JSeparator());
    		//	setMenuItem(menu, "SERVLIST");
    		//	setMenuItem(menu, "SQUERY");
    		//	menu.add(new JSeparator());
    		//	setMenuItem(menu, "WHO");
    		//	setMenuItem(menu, "WHOIS");
    		//	setMenuItem(menu, "WHOWAS");
    		menu.setEnabled(false);
	
    		return menu;
    }

    private JMenu getChannelMenu() {
    		JMenu menu = new JMenu(getMessage("channel"));
	
    		setMenuItem(menu, "PART").addActionListener(new PartActionImpl());
    		//	setMenuItem(menu, "MODE");
    		setMenuItem(menu, "TOPIC...").addActionListener(new TopicActionImpl());
    		//	setMenuItem(menu, "INVITE");
    		//	setMenuItem(menu, "KICK");
    		menu.setEnabled(false);
	
    		return menu;
    }

    /*
    private JMenu getOperMenu() {
	JMenu menu = new JMenu("Oper Command");
	
	setMenuItem(menu, "SQUIT");
	setMenuItem(menu, "KILL");
	menu.setEnabled(false);
	
	return menu;
    }
    */

    private JMenuItem setMenuItem(JMenu menu, String text) {
    		JMenuItem item = new JMenuItem(text);
	
    		menu.add(item);

    		return item;
    }
    
    public void setConnected (boolean connected) {
	//	connectionItem.setEnabled(connected);
	disconnectionItem.setEnabled(connected);
	commandMenu.setEnabled(connected);
	channelMenu.setEnabled(connected);
	//	operMenu.setEnabled(connected);
    }
  
}
