package jp.sourceforge.jirc;
 
import java.awt.*;
    
import javax.swing.*;

import jp.sourceforge.jirc.net.*;

public class Window extends AbstractClient {
    private final String CRLF = "\r\n";

    private ClientComponent clientComp;
    private Frame frame;
    private ConnectionDialog connectionDialog = null;

    private Channel nowChannel = null;

    public Window() {
    		super();
    }

    public Window(JApplet applet) {
    		super();
    		applet.setSize(640, 480);  // magic number
    		clientComp.init(applet.getContentPane());
    }

    void AWTFrame() {
    		AWTFrame awtFrame = new AWTFrame(this);
    		frame = awtFrame;
    		clientComp = awtFrame;
	
    		clientComp.addDefaultWindowListener();
    		frame.setSize(640, 480);  // magic number
    		clientComp.init(awtFrame);
    }

    void SwingFrame() {
    		SwingFrame swingFrame = new SwingFrame(this);
    		frame = swingFrame;
    		clientComp = swingFrame;
	
    		clientComp.addDefaultWindowListener();
    		frame.setSize(640, 480);  // magic number
    		clientComp.init(swingFrame.getContentPane());
    }
    
    public void Connection() {
    		if(connectionDialog == null) {
    			new ConnectionDialog(this);
    		} else {
    			connectionDialog.show();
    		}
    }
    
    public IRC Connection(String server, int port, String nick, String user, String passwd,
    		String channels) {
    		IRC irc = Connection(server, port, nick, user, passwd);

    		if(irc != null) {
//    			while(true) {
//    				if(irc.isConnected()) {
    					super.setJOIN(channels);
//    					break;
//    				}
//    			}
    		}

    		return irc;
    }

    public IRC Connection(String server, int port, String nick, String user, String passwd) {    	
    		IRC irc = null;
    		try {
    			irc = super.Connection(server, port, nick, user, passwd);
    			frame.validate();
    		} catch(Exception e) {
    			setException(e);
    			super.close();
    		} finally {
    			if(irc != null) {
    				setConnection(irc, true);
    			}
    		}
    		return irc;
    	}

    public void writeMessage(String channel, String message) {
    		try {
    			super.writeMessage(channel, message);
    		} catch(Exception e) {
    			setException(e);
    			super.close();
    		}
    }

    void setNowChannel(Channel channel) {
    		this.nowChannel = channel;
    }
    
    Channel getNowChannel() {
    		return nowChannel;
    }

    public IRC getIRC() {
    		if(nowChannel == null) {
    			return null;
    		}

    		return nowChannel.getIRC();
    }

    public void setConnection(IRC irc, boolean flag) {
    		clientComp.setConnection(irc, flag);
    }

    public Channel makeChannel(IRC irc, String name) {
    		Channel channel = clientComp.makeChannel(irc, name);
    		frame.validate();

    		return channel;
    }

    public Channel makeServerChannel(IRC irc, String name) {
    		Channel channel = clientComp.makeServerChannel(irc, name);
    		frame.validate();

    		return channel;
    }

    public void removeChannel(Channel channel) {
    		clientComp.removeChannel(channel);
    }

    public void removeUser(IRC irc, String nick, String message) {
    		clientComp.removeUser(irc, nick, message);
    }
    
    public void changeUser(IRC irc, String oldNick, String newNick) {
		clientComp.changeUser(irc, oldNick, newNick);
    }

    public void append(Channel channel, String string) {
    		clientComp.append(channel, string);
    }

    public void exit() {
    		if(isConnected()) {
    			Disconnection();
    			frame.dispose();
    			frame.hide();
    		} else {
    			System.exit(0);
    		}
    }

    void setTopic() {
    		clientComp.setTopic();
    }

    public void show() {
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
//	    Dimension windowSize = frame.getSize();
	    frame.setLocation((screenSize.width - 640) / 2, (screenSize.height - 480) / 2);

	    frame.validate();
	    frame.show();
    }

    public void setException(Exception e) {
    		JOptionPane.showMessageDialog(null, e.getLocalizedMessage());
    		super.setException(e);
    }
}
