package jp.sourceforge.jirc;

import java.util.Hashtable;
import java.util.Enumeration;

import javax.swing.tree.*;

import jp.sourceforge.jirc.net.*;

class WindowTreeModel extends DefaultTreeModel {
    private static MutableTreeNode rootNode = new DefaultMutableTreeNode();

    private Hashtable serverTable = new Hashtable();

    private SwingFrame window;

    public WindowTreeModel(SwingFrame window) {
    		super(rootNode);
    		this.window = window;
    }
    
    synchronized void addParentWindow(SwingChannel cw) {
    		rootNode.insert(cw.getTreeNode(), rootNode.getChildCount());
    		serverTable.put(cw.toString(), cw);
    		reload(rootNode);
    }

    synchronized void removeParentWindow() {
    		for(Enumeration e = serverTable.keys(); e.hasMoreElements();) {
    			String name = (String )e.nextElement();
    			SwingChannel cw = (SwingChannel )serverTable.get(name);
    			if(!cw.getIRC().isConnected()) {
    				rootNode.remove(cw.getTreeNode());
    				serverTable.remove(cw);
    			}
    		}
    		reload(rootNode);
    }
    
    private MutableTreeNode getServerNode(SwingChannel channel) {
		SwingChannel serverChannel = (SwingChannel )serverTable.get(channel.getIRC().getServer());

		return serverChannel.getTreeNode();
    }

    synchronized void addChannel(SwingChannel cw) {
    		MutableTreeNode serverNode = getServerNode(cw);
    		serverNode.insert(cw.getTreeNode(), serverNode.getChildCount());
    		reload(serverNode);
    }

    void removeChannel(SwingChannel channel) {
    		MutableTreeNode treeNode = channel.getTreeNode();
    		if(treeNode != null) {
        		MutableTreeNode serverNode = getServerNode(channel);
        		serverNode.remove(treeNode);

        		SwingChannel cw = (SwingChannel )serverTable.get(channel);
        		if(cw == null) {
        			cw = (SwingChannel )channel;
        		}
        		window.removeWindow(cw);

        		reload(rootNode);
    		}
    }

    Channel getNowServer() {
    		Channel nowChannel = window.getNowChannel();
    		Channel channel = (Channel ) serverTable.get(nowChannel.toString());

    		if(channel == null) {
    			channel = (Channel ) serverTable.get(nowChannel.getIRC().getServer());
    		}

    		return channel;
    }
    
    void addUser(SwingChannel channel, String nick) {
    		MutableTreeNode treeNode = channel.getTreeNode();

    		if(!hasUser(treeNode, nick)) {
    			treeNode.insert(new DefaultMutableTreeNode(nick),treeNode.getChildCount());
    			reload(treeNode);
    		}
    }

    private boolean hasUser(MutableTreeNode treeNode, String nick) {
		Enumeration e = treeNode.children();
		if(e != null) {
			while(e.hasMoreElements()) {
				MutableTreeNode childNode = (MutableTreeNode )e.nextElement();
				if(childNode.toString().equals(nick)) {
					return true;
				}
			}
		}

		return false;
    }

    boolean removeUser(SwingChannel channel, String nick) {
    		MutableTreeNode treeNode = channel.getTreeNode();
    		if(treeNode == null) {
    			return false;
    		}

    		MutableTreeNode userNode = getUserNode(treeNode, nick);
    		if(userNode != null) {
    			treeNode.remove(userNode);
    			reload(treeNode);
    			return true;
    		} else {
    			return false;
    		}
    }

    void changeUserNode(MutableTreeNode channel, MutableTreeNode oldNick, String newNick) {
    		MutableTreeNode node = new DefaultMutableTreeNode(newNick);

    		channel.remove(oldNick);
    		channel.insert(node, channel.getChildCount());
    		reload(channel);
    }

    MutableTreeNode getUserNode(MutableTreeNode treeNode,String nick) {
    		Enumeration e = treeNode.children();
    		while(e.hasMoreElements()) {
    			MutableTreeNode childNode = (MutableTreeNode )e.nextElement();
    			String user = childNode.toString();
    			if(user.equals(nick) || user.equals("@" + nick) || user.equals("+" + nick)) {
    				return childNode;
    			}
    		}
    		return null;
    }

    void setUserMode(SwingChannel channel, String nick, String mode, boolean flag) {
    		MutableTreeNode treeNode = channel.getTreeNode();
    		MutableTreeNode childNode = getUserNode(treeNode, nick);
    		if(childNode != null) {
    			if(flag) {
    				nick = mode + nick;
    			}
    			changeUserNode(treeNode, childNode, nick);
    		}
    }
}
