package jp.sourceforge.jirc.net;

import jp.sourceforge.jirc.text.IRCMessageFormat;

abstract public class AbstractClient implements Client {
    private String ENC = "JIS";

    public IRC Connection(String server, int port, String nick, String user, String passwd) throws Exception {
    		IRC irc = null;
    		try {
    			irc = new IRC(server, port, nick, user, passwd);
    			irc.addReadMessageListener(this);
    			irc.start();
    		} catch (Exception e) {
    			irc = null;
    			throw e;
    		}

    		return irc;
    }

    abstract public IRC getIRC();
    
    public boolean isConnected() {
    		IRC irc = getIRC();

    		if(irc == null) {
    			return false;
    		}

    		return irc.isConnected();
    }
    
    public void Disconnection() {
    		IRC irc = getIRC();

    		try {
    			if(irc.isConnected()) {
    				irc.QUIT();
    			}
    		} catch(Exception e) {
    			e.printStackTrace();
    		}
    }

    public void close() {
    		IRC irc = getIRC();

    		if(irc != null) {
    			irc.close();
    		}
    }

    public final void setENC(String encoding) {
    		this.ENC = encoding;
    }
    
    public final String getENC() {
    		return ENC;
    }

    public void writeMessage(String channel, String message) throws Exception {
    		IRC irc = getIRC();

    		if (message.startsWith("/")) {
    			message = message.substring(message.indexOf("/") + 1);
    			irc.directMessage(toENC(message));
    		} else {
    			if (channel != null) {
    				irc.PRIVMSG(toENC(channel), toENC(message));

    				String[] args = {irc.getNick(), channel, message};
    				irc.getChannel(channel).append(IRCMessageFormat.getPrivmsgMessage(args));
    			} 
    		}
    }
    
    public String toENC(String string) {
    		try {
    			return new String(string.getBytes(ENC));
    		} catch (Exception e) {
    			System.out.println(string);
    			e.printStackTrace();
    			return string;
    		}
    }
    
    public String encToUNICODE(String string) {
    		try {
    			return new String(string.getBytes(), ENC);
    		} catch (Exception e) {
    			System.out.println(string);
    			e.printStackTrace();
    			return string;
    		}
    }

    public final void setJOIN(String channel) {
    		IRC irc = getIRC();

    		if(!(channel == null)) {
    			try {
    				irc.JOIN(toENC(channel));
    			} catch(Exception e) {
    				Disconnection();
    			}
    		}
    }

    public final void setPART(Channel channel) {
    		try {
    			channel.getIRC().PART(toENC(channel.toString()));
    		} catch(Exception e) {
    			Disconnection();
    		}
    }

    public void setTOPIC(Channel channel, String topic) {
    		if(!(topic == null)) {
    			try {
    				channel.getIRC().TOPIC(channel.toString(), toENC(topic));
    			} catch(Exception e) {
    				Disconnection();
    			}
    		}
    }

    public void setNICK(String nick){
    		IRC irc = getIRC();

    		if(nick != null) {
    			try {
    				irc.NICK(nick);
    			} catch(Exception e) {
    				Disconnection();
    			}
    		}
    }
    
    public void setException(Exception e) {
    		e.printStackTrace();
    }
}
