package jp.sourceforge.jirc.net;

import java.io.*;

import java.net.Socket;
import java.net.SocketException;

import java.util.*;
import java.text.*;

import jp.sourceforge.jirc.*;

class IRCIO extends Thread {
    private IRC irc;
    private BufferedReader  is = null;
    private IRCIOWriter     os = null;
    private BufferedWriter log = null;

    private Socket socket = null;
    private boolean runnable = false;
    
    private SimpleDateFormat formater = null;

    private final String CRLF = "\r\n";
    private String prevDate = null;
    
    IRCIO(IRC irc, String server) throws Exception {
    		this(irc, server, 6667);
    }
    
    IRCIO(IRC irc, String server, int port) throws Exception {
    		super("IRCIO");

    		if(irc != null) {
    			this.irc = irc;
    			runnable = true;
    		}
    		
    		String logDirectory = jIRCPreferences.getLogDirectoryProperty();
    		File file = new File(logDirectory);
    		if(file.isDirectory() && file.canWrite()) {
    			formater = new SimpleDateFormat();
    			formater.applyPattern("yyyyMMddHHmmss");
    			log = new BufferedWriter(new FileWriter(new File(logDirectory, formater.format(new Date())), true));
    			formater.applyPattern("HH:mm");
    			
        		prevDate = formater.format(new Date());
        		write("="+ prevDate);
    		} else {
    			System.out.println(file + " :Log file cant write file.");
    		}

    		open(server, port);
    	}
    
    void open(String server, int port) throws Exception {
       	try {
       		this.socket = new Socket(server, port);
       		/*
       		 System.out.println("SO_KEEPALIVE is " + s.getKeepAlive());
       		 System.out.println("SO_RCVBUF  is " + s.getReceiveBufferSize());
       		 System.out.println("SO_SNDBUF  is " + s.getSendBufferSize());
       		 System.out.println("SO_LINGER  is " + s.getSoLinger());
       		 System.out.println("SO_TIMEOUT is " + s.getSoLinger());
       		 System.out.println("TCP_NODELAY is " + s.getTcpNoDelay());
       		 */
       		is = new BufferedReader(new InputStreamReader(socket.getInputStream()));
       		os = new IRCIOWriter(irc, socket.getOutputStream());
       		os.start();
       	} catch (Exception e) {
       		irc.setException(e);
       		irc.close();
       		socket = null;
       		is = null;
       		os = null;
	    
       		throw e;
       	}
    }
    
    void close() {
    		runnable = false;
    		try {
    			if(is != null) {
    				is.close();
    			}

    			if(os != null) {
    				os.close();
    				os.join();
    			}
    			
    			if(log != null) {
    				log.flush();
    				log.close();
    			}
    		} catch (Exception e) {
    			e.printStackTrace();
    		} finally {
    			is = null;
    			os = null;
    		}
    }
    
    public void run() {
	//	System.out.println("#IRCIO RUN");
	//	Thread.currentThread().getThreadGroup().list();

    		String string = null;
	
    		while(runnable) {
//    			System.out.println("IRCIO runnable is " + runnable);
    			try {
    				string = is.readLine();

    				if(string == null) {
    					System.out.println("IRCIO is.readline() return null");
//    					close();
    				} else {
    					logWrite(">", string);
    					irc.readMessage(string);
    				}
    			} catch (SocketException e) {
    				System.out.println(string);
    				irc.setException(e);
    				irc.close();
    			} catch (Exception e) {
    				System.out.println(string);
    				irc.setException(e);
    			}
    		}
    }
    
    final void writeMessage(String message) throws IOException {
    		logWrite("<", message);

    		/*
    		if(!message.startsWith("PONG")) {
    			System.out.print(message);
    		}
    		*/

    		if(os != null) {
    			os.writeMessage(message + CRLF);
    		}
    }
    
    private void logWrite(String head, String message) {
    		if(prevDate != null) {
    			String date = formater.format(new Date());
    			if(!prevDate.equals(date)) {
    				write("="+ date);
    				prevDate = date;
    			}
    			write(head + message);
    		}
    }
    
    private void write(String string) {
    		try {
    			log.write(string);
    			log.newLine();
    			log.flush();
    		} catch(Exception e) {
    			irc.setException(e);
    		}
    }
}
