package jp.sourceforge.jirc.net;

import java.io.IOException;
import java.io.BufferedReader;

import java.net.InetAddress;
import java.net.InetSocketAddress;

import java.nio.ByteBuffer;
import java.nio.CharBuffer;

import java.nio.channels.Channels;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;
import java.nio.charset.CharsetEncoder;

class IRCNIO extends Thread {
    private IRC irc;
//    private final String CRLF = "\r\n";
    
    private boolean runnable = false;
    private SocketChannel socketChannel = null;
    private static ByteBuffer buffer = ByteBuffer.allocateDirect(2048);

    private BufferedReader is = null;

    private static Charset charsetJIS = Charset.forName("JIS");
    private static CharsetEncoder encoder = charsetJIS.newEncoder();

    //    private static Charset charsetUTF = Charset.forName("UTF-8");
    //    private static CharsetDecoder decoder = charsetUTF.newDecoder();

    IRCNIO(IRC irc, String server) throws Exception {
	this(irc, server, 6667);
    }
    
    IRCNIO(IRC irc, String server, int port) throws Exception {
	super("IRCIO");

	if(irc != null) {
	    this.irc = irc;
	    runnable = true;
	}
	open(server, port);
    }
    
    void open(String server, int port) throws Exception {
       	try {
	    InetSocketAddress address
		= new InetSocketAddress(InetAddress.getByName(server), port);
	    socketChannel = SocketChannel.open();
	    socketChannel.connect(address);
	    is = new BufferedReader(Channels.newReader(socketChannel, "UTF-8"));

	    /*
	    System.out.println("SO_KEEPALIVE is " + s.getKeepAlive());
	    System.out.println("SO_RCVBUF  is " + s.getReceiveBufferSize());
	    System.out.println("SO_SNDBUF  is " + s.getSendBufferSize());
	    System.out.println("SO_LINGER  is " + s.getSoLinger());
	    System.out.println("SO_TIMEOUT is " + s.getSoLinger());
	    System.out.println("TCP_NODELAY is " + s.getTcpNoDelay());
	    */
	} catch (Exception e) {
	    System.out.println(e);
	    irc.setException(e);
	    irc.close();
	    socketChannel  = null;
	    
	    throw e;
	}
    }
    
    synchronized final void close() {
	runnable = false;
    }
    
    public void run() {
	/*
	System.out.println("#IRCIO RUN");
	Thread.currentThread().getThreadGroup().list();
	*/

	String string = "";
	
	while(runnable) {
	    try {
		string = is.readLine();
	    } catch (Exception e) {
		irc.setException(e);
	    }
	    if(string == null) {
		close();
	    } else if(string == "") {
		close();
	    } else {
		try {
		    irc.readMessage(string);
		} catch(NullPointerException e) {
		    System.out.println(string);
		    irc.setException(e);
		}
	    }
	}
    }
    
    final synchronized void writeMessage(String message) throws IOException {
	// debug
	if(!message.startsWith("PONG")) {
	    System.out.print(message);
	}

	socketChannel.write(encoder.encode(CharBuffer.wrap(message)));
    }
}
