package jp.sourceforge.jirc.net;

import java.io.*;
import java.util.*;

class NUMERICREPLY implements Command {
    private final IRC irc;

    public NUMERICREPLY(IRC irc) {
	this.irc = irc;
    }

    private final void append(String channelname, String message) {
    		append(irc.getChannel(channelname), message);
    }
    
    private final void appendCommandMessage(String channelname, String message) {
		appendCommandMessage(irc.getChannel(channelname), message);
    }
    
    private final void appendErrorMessage(String channelname, String message) {
		appendErrorMessage(irc.getChannel(channelname), message);
    }

    private final void append(Channel channel, String message) {
    		irc.getClient().append(channel, message);
    		channel.append(message);
    }
    
    private final void appendCommandMessage(Channel channel, String message) {
    		irc.getClient().append(channel, message);
    		channel.appendCommandMessage(message);
    	}
    
    private final void appendErrorMessage(Channel channel, String message) {
    		irc.getClient().append(channel, message);
    		channel.appendErrorMessage(message);
    }

    private final ChannelMode getMode(String channelname) {
	return irc.getChannel(channelname).getMode();
    }

    String removeColon(String string) {
	return CommandMap.removeColon(string);
    }

    String getNick(String prefix){
	return CommandMap.getNick(prefix);
    }

    public void execute(String params) {};

    public void execute(String command, String params) {
    		Integer num = new Integer(command);

    		params = params.substring(params.indexOf(' ') +1);
    		StringTokenizer st = new StringTokenizer(params, " ",false);

       	String channelname = "";
       	String mode = "";
       	Channel channel = null;

       	switch(num.intValue()) {
       		case 4:
       			irc.setConnected(true);
       		case 1:
       		case 2:
       		case 3:
       		case 5:
       		case 6:
       		case 7:
       		case 8:
       		case 9:
       			append(irc.getServer(), command + " " + removeColon(params));
       			break;
       		case 311:
       		case 312:
       		case 315:
       		case 317:
       		case 318:
       		case 319:
       			st.nextToken();
       			append(irc.getServer(),
       					command + " " + removeColon(st.nextToken(CRLF).trim()));
       			break;
       		case 324:
       			channel = irc.getChannel(st.nextToken());
       			if(channel != null) {
       				channel.setMode(st.nextToken(CRLF).trim());
       			}
       			break;
       		case 332:
       			channel = irc.getChannel(st.nextToken());
       			if(channel != null) {
       				channel.setTopic(removeColon(st.nextToken(CRLF).trim()));
       			}
       			channel = null;
       			break;
       		case 333:
       			// channel nick time
       			break;
       		case 353:
       			mode = st.nextToken();// channel mode see rfc2812 num353
       			channelname = st.nextToken();
       			channel = irc.getChannel(channelname);
       			String names = removeColon(st.nextToken(CRLF).trim());

       			if(channel != null) {
       				if(mode.equals("=")) {
       					channel.getMode().setPublic(true);
       				} else if (mode.equals("*")) {
       					channel.getMode().setPrivate(true);
       				} else if (mode.equals("@")) {
       					channel.getMode().setSecret(true);
       				}
		
       				StringTokenizer namest = new StringTokenizer(names, " ",false);
       				while(namest.hasMoreTokens()) {
       					channel.addUser(namest.nextToken());
       				}
       			} else {
       				channelname = irc.getServer();
       				names = command + " " + names;
       			}
       			append(channelname, names);
       			break;
       		case 366:
       			break;
       		case 375:
       		case 376:
       			append(irc.getServer(), command + " " + removeColon(params));
       			break;
       		case 403:
       			channelname = st.nextToken();
       			channel = irc.getChannel(channelname);
       			if(channel != null) {
       				irc.removeChannel(channel);
       			} else {
       				appendErrorMessage(irc.getServer(), command + " " + removeColon(params));
       			}
       			break;
       		case 433:
       			String newNick = st.nextToken() + "_";
       			irc.setNick(newNick);
       			try {
       				irc.NICK(newNick);
       			} catch(IOException e) {
       				e.printStackTrace();
       			}
       			break;
       		default:
       			append(irc.getServer(), command + " " + removeColon(params));
       	}
    }
}
